"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lram = exports.helloWorld = exports.Lram = void 0;
var code_1 = require("./plug/dynamic/code");
var img_1 = require("./plug/dynamic/img");
var txt_1 = require("./plug/static/txt");
var table_1 = require("./plug/dynamic/table");
var katex_1 = require("./plug/dynamic/katex");
var color_1 = require("./plug/inline/color");
var green_1 = require("./plug/inline/green");
var title_1 = require("./plug/static/title");
var token_1 = require("./model/token/token");
var list_1 = require("./plug/dynamic/list");
var parserMap = new Map();
var subParserMap = new Map();
var renderMap = new Map();
var inlineParserArr = new Array();
var inlineRenderMap = new Map();
var imgTagMap = new Map();
var aliasMap = new Map();
console.log(imgTagMap);
console.log(aliasMap);
var Lram = /** @class */ (function () {
    function Lram() {
        this.use(title_1.titlePlug);
        this.use(katex_1.katexPlug);
        this.use(code_1.codePlug);
        this.use(table_1.tablePlug);
        this.use(txt_1.txtPlug);
        this.use(img_1.imgPlug);
        this.use(list_1.listPlug);
        this.useInline(color_1.colorPlug);
        this.useInline(green_1.greenPlug);
    }
    Lram.prototype.useInline = function (f) {
        if (f.parser !== undefined) {
            inlineParserArr.push(f.parser);
        }
        f.render.map(function (render) {
            inlineRenderMap[render.code] = render.render;
        });
    };
    Lram.prototype.use = function (f) {
        if (f.parser !== undefined) {
            parserMap[f.code] = f.parser;
        }
        f.render.map(function (render) {
            renderMap[render.code] = render.render;
            subParserMap[render.code] = render.subParserType;
        });
    };
    Lram.prototype.render = function (str) {
        return coreTran(str, new token_1.BaseToken('init'));
    };
    return Lram;
}());
exports.Lram = Lram;
function inlineCoreTran(line) {
    var i = 0;
    var html = '';
    while (true) {
        var flag = false;
        for (var _i = 0, inlineParserArr_1 = inlineParserArr; _i < inlineParserArr_1.length; _i++) {
            var lineP = inlineParserArr_1[_i];
            var ds = lineP(line);
            if (ds.match) {
                flag = true;
                for (var _a = 0, _b = ds.tokens; _a < _b.length; _a++) {
                    var t = _b[_a];
                    var token = t;
                    if (token.code === 's-txt') {
                        i++;
                        if (i === 3) {
                            throw '100';
                        }
                        html += inlineCoreTran(token.data);
                    }
                    else {
                        html += inlineRenderMap[token.code](token.data);
                    }
                }
                return html;
            }
        }
        if (!flag) {
            html += line;
            break;
        }
    }
    return html;
}
function coreRender(t, context) {
    var token = t;
    if (token.code === 's-txt') {
        return inlineCoreTran(token.data);
    }
    return renderMap[token.code](token, context, coreTran);
}
function coreTran(lineData, preToken) {
    if (preToken.code === 's-txt') {
        return inlineCoreTran(lineData);
    }
    var lines = lineData.trim().split('\n');
    var html = '';
    while (lines.length > 0) {
        var flag = false;
        var parserFunCodes = Object.keys(parserMap);
        for (var _i = 0, parserFunCodes_1 = parserFunCodes; _i < parserFunCodes_1.length; _i++) {
            var p = parserFunCodes_1[_i];
            if (preToken.code !== 'init' && subParserMap[preToken.code].indexOf(p) < 0) {
                continue;
            }
            //深拷贝
            var v = JSON.parse(JSON.stringify(lines));
            var ds = parserMap[p](v);
            if (ds.line > 0) {
                flag = true;
                lines.splice(0, ds.line);
                ds.tokens.map(function (token) {
                    html += coreRender(token, null);
                });
                break;
            }
        }
        if (!flag) {
            var string = lines[0];
            if (string.substr(0, 2) !== '//') {
                html += inlineCoreTran(string);
            }
            lines.shift();
        }
    }
    return "<div class=\"lram\">".concat(html, "</div>");
}
var helloWorld = function () {
    return 'Howdy!';
};
exports.helloWorld = helloWorld;
exports.lram = new Lram();
//# sourceMappingURL=index.js.map